// **********************************************************************
// COPYRIGHT NOTICE AND DISCLAIMER
//
// Copyright (c) Erik Meijering. All rights reserved.
//
// Permission to copy and use this software and accompanying
// documentation for educational, research, and not-for-profit
// purposes, without fee and without a signed licensing agreement,
// is hereby granted, provided that the above copyright notice, this
// paragraph and the following two paragraphs appear in all copies.
// The copyright holder is free to make upgraded or improved versions
// of the software available for a fee or commercially only.
//
// IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE TO ANY PARTY FOR
// DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
// OF ANY KIND WHATSOEVER, ARISING OUT OF THE USE OF THIS SOFTWARE
// AND ITS DOCUMENTATION, EVEN IF HE HAS BEEN ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.
//
// THE COPYRIGHT HOLDER SPECIFICALLY DISCLAIMS ANY WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
// SOFTWARE AND ACCOMPANYING DOCUMENTATION IS PROVIDED "AS IS".
// THE COPYRIGHT HOLDER HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
// SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
//
// **********************************************************************

package imagescience.images;

/** Contains size information about the borders of (at most) 4D data
    objects. */
public final class Borders {

    /** The border size in the x-dimension. */
    public final int x;

    /** The border size in the y-dimension. */
    public final int y;

    /** The border size in the z-dimension. */
    public final int z;

    /** The border size in the t-dimension. */
    public final int t;

    /** Default constructor. Each of the borders is set to 0. */
    public Borders() {
	this.x = 0;
	this.y = 0;
	this.z = 0;
	this.t = 0;
    }

    /** One-dimensional constructor. The border size in the x-dimension
        is set to the given value. The y-, z-, and t-borders are set
        to 0.

	@exception IllegalArgumentException if the parameter is
	negative.
    */
    public Borders(final int x) {
	if (x < 0) throw new IllegalArgumentException("Negative border size");
	this.x = x;
	this.y = 0;
	this.z = 0;
	this.t = 0;
    }

    /** Two-dimensional constructor. The border sizes in the x- and
        y-dimension are set to the given values. The border sizes in
        the z- and t-dimension are set to 0.

	@exception IllegalArgumentException if any of the parameters is
	negative.
    */
    public Borders(final int x, final int y) {
	if (x < 0 || y < 0) throw new IllegalArgumentException("Negative border size(s)");
	this.x = x;
	this.y = y;
	this.z = 0;
	this.t = 0;
    }

    /** Three-dimensional constructor. The border sizes in the x-, y-,
        and z-dimension are set to the given values. The border size
        in the t-dimension is set to 0.

	@exception IllegalArgumentException if any of the parameters is
	negative.
    */
    public Borders(final int x, final int y, final int z) {
	if (x < 0 || y < 0 || z < 0) throw new IllegalArgumentException("Negative border size(s)");
	this.x = x;
	this.y = y;
	this.z = z;
	this.t = 0;
    }

    /** Four-dimensional constructor. The border sizes in the x-, y-,
        z-, and t-dimension are set to the given values.

	@exception IllegalArgumentException if any of the parameters is
	negative.
    */
    public Borders(final int x, final int y, final int z, final int t) {
	if (x < 0 || y < 0 || z < 0 || t < 0) throw new IllegalArgumentException("Negative border size(s)");
	this.x = x;
	this.y = y;
	this.z = z;
	this.t = t;
    }

    /** Returns a copy of the current <tt>Borders</tt> object. */
    public Borders duplicate() { return new Borders(x,y,z,t); }

    /** Returns the border sizes as a <tt>Dimensions</tt> object. */
    public Dimensions dimensions() { return new Dimensions(x,y,z,t); }

    /** Indicates whether or not the given border sizes are equal to the
        current border sizes.

	@return <tt>true</tt> if each of the current border sizes is
	equal to the corresponding parameter border size; <tt>false</tt>
	if this is not the case.

	@exception NullPointerException if the parameter is <tt>null</tt>.  */
    public boolean equals(Borders borders) {
	if (x==borders.x && y==borders.y && z==borders.z && t==borders.t) return true;
	return false;
    }

}
