// **********************************************************************
// COPYRIGHT NOTICE AND DISCLAIMER
//
// Copyright (c) Erik Meijering. All rights reserved.
//
// Permission to copy and use this software and accompanying
// documentation for educational, research, and not-for-profit
// purposes, without fee and without a signed licensing agreement,
// is hereby granted, provided that the above copyright notice, this
// paragraph and the following two paragraphs appear in all copies.
// The copyright holder is free to make upgraded or improved versions
// of the software available for a fee or commercially only.
//
// IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE TO ANY PARTY FOR
// DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
// OF ANY KIND WHATSOEVER, ARISING OUT OF THE USE OF THIS SOFTWARE
// AND ITS DOCUMENTATION, EVEN IF HE HAS BEEN ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.
//
// THE COPYRIGHT HOLDER SPECIFICALLY DISCLAIMS ANY WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
// SOFTWARE AND ACCOMPANYING DOCUMENTATION IS PROVIDED "AS IS".
// THE COPYRIGHT HOLDER HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
// SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
//
// **********************************************************************

package imagescience.images;

/** Contains size information about (at most) 4D pixels. */
public final class PixelSizes {

    /** The pixel size in the x-dimension. */
    public final float x;

    /** The pixel size in the y-dimension. */
    public final float y;

    /** The pixel size in the z-dimension. */
    public final float z;

    /** The pixel size in the t-dimension. */
    public final float t;

    /** Default constructor. The pixel size is set to 1.0 in all dimensions. */
    public PixelSizes() {
	this.x = 1.0f;
	this.y = 1.0f;
	this.z = 1.0f;
	this.t = 1.0f;
    }

    /** One-dimensional constructor. The pixel size in the x-dimension
        is set to the given value. The pixel sizes in the y-, z-, and
        t-dimension are set to 1.0.

	@exception IllegalArgumentException if the parameter is negative
	or zero.
    */
    public PixelSizes(final float x) {
	if (x <= 0.0f) throw new IllegalArgumentException("Argument negative or zero");
	this.x = x;
	this.y = 1.0f;
	this.z = 1.0f;
	this.t = 1.0f;
    }

    /** Two-dimensional constructor. The pixel sizes in the x- and
        y-dimension are set to the given values. The pixel sizes in
        the z- and t-dimension are set to 1.0.

	@exception IllegalArgumentException if any of the parameters is
	negative or zero.
    */
    public PixelSizes(final float x, final float y) {
	if (x <= 0 || y <= 0) throw new IllegalArgumentException("Argument(s) negative or zero");
	this.x = x;
	this.y = y;
	this.z = 1.0f;
	this.t = 1.0f;
    }

    /** Three-dimensional constructor. The pixel sizes in the x-, y-,
        and z-dimension are set to the given values. The pixel size
        in the t-dimension is set to 1.0.

	@exception IllegalArgumentException if any of the parameters is
	negative or zero.
    */
    public PixelSizes(final float x, final float y, final float z) {
	if (x <= 0 || y <= 0 || z <= 0) throw new IllegalArgumentException("Argument(s) negative or zero");
	this.x = x;
	this.y = y;
	this.z = z;
	this.t = 1.0f;
    }

    /** Four-dimensional constructor. The pixel sizes in the x-, y-, z-,
        and t-dimension are set to the given values.

	@exception IllegalArgumentException if any of the parameters is
	negative or zero.
    */
    public PixelSizes(final float x, final float y, final float z, final float t) {
	if (x <= 0 || y <= 0 || z <= 0 || t <= 0) throw new IllegalArgumentException("Argument(s) negative or zero");
	this.x = x;
	this.y = y;
	this.z = z;
	this.t = t;
    }

    /** Returns a copy of the current <tt>PixelSizes</tt> object. */
    public PixelSizes duplicate() { return new PixelSizes(x,y,z,t); }

    /** Indicates whether or not the given pixel sizes are equal to the
        current pixel sizes.

	@return <tt>true</tt> if each of the current pixel sizes is
        equal to the corresponding parameter pixel size; <tt>false</tt>
        if this is not the case.

	@exception NullPointerException if the parameter is <tt>null</tt>.
    */
    public boolean equals(PixelSizes pixelsizes) {
	if (x==pixelsizes.x && y==pixelsizes.y && z==pixelsizes.z && t==pixelsizes.t) return true;
	return false;
    }

}
