//**********************************************************************
//COPYRIGHT NOTICE AND DISCLAIMER
//
//Copyright (c) Erik Meijering. All rights reserved.
//
//Permission to copy and use this software and accompanying
//documentation for educational, research, and not-for-profit
//purposes, without fee and without a signed licensing agreement,
//is hereby granted, provided that the above copyright notice, this
//paragraph and the following two paragraphs appear in all copies.
//The copyright holder is free to make upgraded or improved versions
//of the software available for a fee or commercially only.
//
//IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE TO ANY PARTY FOR
//DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
//OF ANY KIND WHATSOEVER, ARISING OUT OF THE USE OF THIS SOFTWARE
//AND ITS DOCUMENTATION, EVEN IF HE HAS BEEN ADVISED OF THE
//POSSIBILITY OF SUCH DAMAGE.
//
//THE COPYRIGHT HOLDER SPECIFICALLY DISCLAIMS ANY WARRANTIES,
//INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
//MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
//SOFTWARE AND ACCOMPANYING DOCUMENTATION IS PROVIDED "AS IS".
//THE COPYRIGHT HOLDER HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
//SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
//
//**********************************************************************
package neuronJgui;

import ij.IJ;
import ij.gui.GUI;
import imagescience.utilities.FMath;

import java.awt.Button;
import java.awt.Checkbox;
import java.awt.Choice;
import java.awt.Component;
import java.awt.Dialog;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Label;
import java.awt.Panel;
import java.awt.TextField;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;

public final class ParametersDialog extends Dialog implements ActionListener, FocusListener {

    private final TextField scaleField;
    private final TextField gammaField;
    private final TextField xsizeField;
    private final TextField ysizeField;
    private final TextField unitsField;

    private final Choice snapChoice;
    private final Choice dijkChoice;
    private final Choice smoothChoice;
    private final Choice sampleChoice;

    private final Checkbox autosaveCheckbox;
    private final Checkbox debugCheckbox;

    private final Button saveButton;
    private final Button okayButton;
    private final Button cancelButton;

    private final GridBagConstraints c = new GridBagConstraints();
    private final GridBagLayout grid = new GridBagLayout();

    private boolean bFirstTextField = true;
    private boolean bScaleChanged = false;
    private boolean bGammaChanged = false;

    // Builds the dialog for setting the parameters.
    ParametersDialog() {

	super(IJ.getInstance(),"NeuronJ: Parameters",true);
	setLayout(grid);

	// Add parameters:
	scaleField = addTextField("Hessian smoothing scale:",IJ.d2s(NJ.scale,FMath.decimals(NJ.scale)));
	gammaField = addTextField("Cost weight factor:",IJ.d2s(NJ.gamma,FMath.decimals(NJ.gamma)));

        snapChoice = addChoice("Snap window size:");
	final int maxsnapsize = 19;
        for (int i=1; i<=maxsnapsize; i+=2) snapChoice.addItem(i+" x "+i);
        snapChoice.select(NJ.snaprange);

        dijkChoice = addChoice("Path-search window size:");
	final int maxdijksize = 2500;
        for (int i=100; i<=maxdijksize; i+=100) dijkChoice.addItem(i+" x "+i);
        dijkChoice.select(NJ.dijkrange/100 - 1);

        smoothChoice = addChoice("Tracing smoothing range:");
	final int maxsmoothrange = 10;
        for (int i=0; i<=maxsmoothrange; ++i) smoothChoice.addItem(String.valueOf(i));
        smoothChoice.select(NJ.halfsmoothrange);

        sampleChoice = addChoice("Tracing subsampling factor:");
	final int maxsubsample = 10;
        for (int i=1; i<=maxsubsample; ++i) sampleChoice.addItem(String.valueOf(i));
        sampleChoice.select(NJ.subsamplefactor-1);

	bFirstTextField = true;
	xsizeField = addTextField("Pixel size in x-dimension:",IJ.d2s(NJ.xsize,FMath.decimals(NJ.xsize)));
	ysizeField = addTextField("in y-dimension:",IJ.d2s(NJ.ysize,FMath.decimals(NJ.ysize)));
	unitsField = addTextField("units:",NJ.units);

        c.insets = new Insets(20, 18, 0, 18);
        c.gridx = 0; c.gridy++; c.gridwidth = 3;
        c.anchor = GridBagConstraints.WEST;
        autosaveCheckbox = new Checkbox(" Automatically save tracings");
        grid.setConstraints(autosaveCheckbox, c);
        autosaveCheckbox.setState(NJ.autosave);
        add(autosaveCheckbox);
	c.gridy++; c.insets = new Insets(0, 18, 0, 18);
        debugCheckbox = new Checkbox(" Show debug messages");
        grid.setConstraints(debugCheckbox, c);
        debugCheckbox.setState(NJ.debug);
        add(debugCheckbox);

	// Add Save, Okay, and Cancel buttons:
	final Panel buttons = new Panel();
        buttons.setLayout(new FlowLayout(FlowLayout.CENTER, 5, 0));
        saveButton = new Button("  Save  ");
        saveButton.addActionListener(this);
        okayButton = new Button("   OK   ");
        okayButton.addActionListener(this);
        cancelButton = new Button("Cancel");
        cancelButton.addActionListener(this);
        buttons.add(saveButton);
        buttons.add(okayButton);
        buttons.add(cancelButton);
	c.gridx = 0; c.gridy++; c.gridwidth = 3;
        c.anchor = GridBagConstraints.EAST;
        c.insets = new Insets(25, 10, 12, 10);
        grid.setConstraints(buttons, c);
        add(buttons);

	// Pack and show:
	pack();
	GUI.center(this);
	setResizable(false);
	show();
    }
	
    private Label makeLabel(String label) {
	if (IJ.isMacintosh()) label += "  ";
	return new Label(label);
    }
    
    private float stringToFloat(final String s, final float defval) {
	
	try {
	    final Float f = new Float(s);
	    return f.floatValue();
	}
	catch(NumberFormatException e) {}

        return defval;
    }

    private Choice addChoice(final String label) {

        final Label newLabel = makeLabel(label);
	c.gridx = 0; c.gridy++; c.gridwidth = 1;
        c.anchor = GridBagConstraints.EAST;
        c.insets = new Insets(0, 12, 3, 0);
        grid.setConstraints(newLabel, c);
        add(newLabel);
        final Choice newChoice = new Choice();
        c.gridx = 1; c.gridwidth = 2;
        c.anchor = GridBagConstraints.WEST;
	c.insets = new Insets(0, 0, 3, 13);
        grid.setConstraints(newChoice, c);
        add(newChoice);

	return newChoice;
    }

    private TextField addTextField(final String label, final String value) {

	c.gridx = 0; c.gridy++;
	c.gridwidth = 1;
        c.anchor = GridBagConstraints.EAST;
        if (bFirstTextField) c.insets = new Insets(25, 12, 3, 0);
	else c.insets = new Insets(0, 0, 3, 0);
	final Label newLabel = makeLabel(label);
        grid.setConstraints(newLabel,c);
	add(newLabel);

        c.gridx = 1;
        c.anchor = GridBagConstraints.WEST;
        if (bFirstTextField) c.insets = new Insets(25, 0, 3, 15);
	else c.insets = new Insets(0, 0, 3, 15);
 	final TextField newTextField = new TextField(value, 6);
        grid.setConstraints(newTextField,c);
        newTextField.setEditable(true);
	newTextField.addFocusListener(this);
	add(newTextField);

	bFirstTextField = false;
	return newTextField;
    }

    public void actionPerformed(final ActionEvent e) {

	if (e.getSource() == okayButton) {
	    setParams(); IJ.showStatus("Set parameters");
	} else if (e.getSource() == saveButton) {
	    setParams(); NJ.saveParams(); IJ.showStatus("Saved parameters");
	} else { IJ.showStatus(NJ.COPYRIGHT); }
	
	setVisible(false);
	dispose();
    }

    private void setParams() {
	
	final boolean debug = debugCheckbox.getState();
	if (debug) NJ.debug = true;
	
	NJ.write("Setting parameters...");
	
	float scale = stringToFloat(scaleField.getText(),1.0f);
	if (scale < 1.0f) scale = 1.0f;
	if (scale == NJ.scale) bScaleChanged = false;
	else bScaleChanged = true;
	NJ.scale = scale;
	NJ.write("   Hessian smoothing scale = "+NJ.scale+" pixels");
	
	float gamma = stringToFloat(gammaField.getText(),0.5f);
	if (gamma < 0.0f) gamma = 0.0f;
	else if (gamma > 1.0f) gamma = 1.0f;
	if (gamma == NJ.gamma) bGammaChanged = false;
	else bGammaChanged = true;
	NJ.gamma = gamma;
	NJ.write("   Cost weight factor = "+NJ.gamma);
	
	NJ.snaprange = snapChoice.getSelectedIndex();
	final int snapwinsize = 2*NJ.snaprange + 1;
	NJ.write("   Snap window size = "+snapwinsize+" x "+snapwinsize+" pixels");
	
	NJ.dijkrange = 100*(dijkChoice.getSelectedIndex() + 1);
	NJ.write("   Path-search window size = "+NJ.dijkrange+" x "+NJ.dijkrange+" pixels");
	
	NJ.halfsmoothrange = smoothChoice.getSelectedIndex();
	NJ.write("   Tracing smoothing range = "+NJ.halfsmoothrange+" pixels on both sides");
	
	NJ.subsamplefactor = sampleChoice.getSelectedIndex() + 1;
	NJ.write("   Tracing subsampling factor = "+NJ.subsamplefactor);
	
	NJ.xsize = stringToFloat(xsizeField.getText(),1.0f);
	if (NJ.xsize < 0.0f) NJ.xsize = 0.0f;
	NJ.write("   Pixel size in x-dimension = "+NJ.xsize+" units");
	
	NJ.ysize = stringToFloat(ysizeField.getText(),1.0f);
	if (NJ.ysize < 0.0f) NJ.ysize = 0.0f;
	NJ.write("   Pixel size in y-dimension = "+NJ.ysize+" units");
	
	NJ.units = unitsField.getText();
	NJ.write("   Pixel units = "+NJ.units);
	
	NJ.autosave = autosaveCheckbox.getState();
	if (NJ.autosave) NJ.write("   Automatically saving tracings");
	else NJ.write("   Asking user to save tracings");

	if (debug) NJ.write("   Showing debug messages");
	else NJ.write("   Stop showing debug messages");

	NJ.write("Done");

	NJ.debug = debug;
	NJ.save = true;
    }
    
    boolean scaleChanged() { return bScaleChanged; }

    boolean gammaChanged() { return bGammaChanged; }

    public void focusGained(final FocusEvent e) {
	Component c = e.getComponent();
	if (c instanceof TextField) ((TextField)c).selectAll();
    }
	
    public void focusLost(final FocusEvent e) {
	Component c = e.getComponent();
	if (c instanceof TextField) ((TextField)c).select(0,0);
    }
	

}
